#!BPY

""" Registration info for Blender menus:
Name: 'Ldraw'
Blender: 233
Group: 'Import'
Tip: 'Import Ldraw (LEGO Cad) files.'
"""

#////////////////////////////////////// LDRAW ///////////////////////////////////*
#/////
#/////	SCRIPT: LDRAW_IMPORT.PY
#/////	VERSION: 0.02.00
#/////
#/////	AUTHORS:	Alessandro V. 2004 - alv67@katamail.com
#/////                  Alex Lindeijer
#/////
#/////	REQUIRMENTS:	Blender 2.33+ (don't know if it works in version 2.32)
#/////
#/////  REFERENCES:     LDraw.org Official Specifications
#/////                  - LDraw File Format          (May 29, 2003)
#/////                  - MPD Language Extension     (May 29, 2003)
#/////                  - LDraw Language Extension for Clipping
#/////                                               (Sep 22, 2003)
#////                   - LDraw Color Chart          (Sep 05, 2003)
#/////
#/////	TODO:	        - Correct face normals
#/////                  - Import some meta commands :
#/////                      "0 BFC [CLIP|NOCLIP] [CW|CCW] [INVERTNEXT]"
#/////                  - Read Multi-Part DAT files (MPD extension)
#/////                      "0 FILE model.ldr"  Multi-part DAT or MPD
#/////                  - Import       "Line"  ("2") command (if needed)
#/////                      "Conditiona-line"  ("5") command (if needed)
#/////                  - Improve materials, colours are not completely
#/////                      correct, auto-smooth, better shader, transparency
#/////                  - Restructure code, lot of double code, bit of hack
#/////
#/////	LICENSE:        The script is under GPL and feel free to use it.
#/////
#/////
#/////- REVISION HISTORY --------------------------------------------------------
#/////
#/////  Version 0.02.01 - 09 august 2004
#/////  --------------------------------
#/////  by Alex Lindeijer
#/////    Fixes: - added legocolor to file
#/////
#/////  Version 0.02.00 - 08 august 2004
#/////  --------------------------------
#/////  by Alex Lindeijer
#/////    Fixes: - Correct coords: blender (x,y,z)= ldraw(x,z.-y)
#/////           - no double vertexes
#/////           - every part and model (in PARTS and MODELS directory is an object)
#/////           - mesh for same partnumber is reused.
#/////
#/////  Version 0.01.01 - 04 july 2004
#/////  ------------------------------
#/////  by Alessandro V. 2004 - alv67@katamail.com
#/////    Fixes : - Corrected bad rotation and translation of some objects due
#/////              to a not good code in transposing coordinate system.
#/////              Now object are imported whith original (Ldraw) axes.
#/////     Code : - Introduced the Split() function and restructured software
#/////              using this function and deleting CVertex, CValue classes
#/////
#/////  Version 0.01.00 - 29 june 2004
#/////  ------------------------------
#/////    Initial release by Alessandro V. 2004 - alv67@katamail.com
#/////
#////////////////////////////////////// LDRAW ///////////////////////////////////*


import Blender

from Blender import NMesh, Material, Texture, Scene, Lamp, Image, sys, Mathutils
from Blender.Draw import *
from Blender.BGL import *
from Blender.Mathutils import *
LDrawPath="C:\Ldraw"      # LDRAW base installation directory

EVENT_PATHCHANGE=		1
EVENT_IMPORT=			2
EVENT_IMPORT_MATS=		3
EVENT_QUIT=			4

legocolor={
0: [51,51,51],
1: [0, 51, 178],
2: [0, 127, 51],
3: [249, 233, 153],
4: [ 204,   0,   0],
5: [255, 51, 153],
6: [102, 51, 0],
7: [153, 153, 153],
8: [102, 102,  88],
9: [0, 128, 255],
10: [51, 255, 102],
11: [51, 166, 167],
12: [255, 201, 196],
13: [ 255, 176, 204],
14: [ 255, 229,   0],
15: [ 255, 255, 255],
17: [102,240,153],
18: [204, 142, 104],
20: [224,204,240],
21: [196, 40, 27],
22: [153, 51, 153],
23: [76, 0, 204],
24: [245, 205, 47],
25: [98, 71, 50],
26: [27, 42, 52],
27: [229, 255, 168],
28: [250, 163, 157],
29: [161, 196, 139],
32: [110,110,110],
33: [0,0,153],
34: [0,80,24],
36: [243, 207, 155],
37: [75, 151, 74],
38: [160, 95, 52],
39: [193, 202, 222],
40: [236, 236, 236],
41: [205, 84, 75],
42: [193, 223, 240],
43: [123, 182, 232],
44: [247, 241, 141],
45: [180, 210, 227],
47: [255, 255, 255],
48: [132, 182, 141],
49: [248, 241, 132],
50: [236, 232, 222],
100: [238, 196, 182],
101: [218, 134, 121],
102: [110, 153, 201],
103: [199, 193, 183],
104: [107, 50, 123],
105: [226, 155, 63],
106: [218, 133, 64],
107: [0, 143, 155],
108: [104, 92, 67],
110: [67, 84, 147],
111: [191, 183, 177],
112: [104, 116, 172],
113: [228, 173, 200],
115: [199, 210, 60],
116: [85, 165, 175],
118: [183, 215, 213],
119: [164, 189, 70],
120: [217, 228, 167],
121: [231, 172, 88],
123: [211, 111, 76],
124: [146, 57, 120],
125: [234, 184, 145],
126: [165, 165, 203],
127: [220, 188, 129],
128: [174, 122, 89],
131: [156, 163, 168],
133: [213, 115, 61],
134: [216, 221, 86],
135: [116, 134, 156],
136: [135, 124, 144],
137: [224, 152, 100],
138: [149, 138, 115],
140: [32, 58, 86],
141: [39, 70, 44],
143: [207, 226, 247],
145: [121, 136, 161],
146: [149, 142, 163],
147: [147, 135, 103],
148: [87, 88, 87],
149: [22, 29, 50],
150: [171, 173, 172],
151: [120, 144, 129],
153: [149, 121, 118],
154: [123, 46, 47],
157: [255, 246, 123],
158: [225, 164, 194],
168: [117, 108, 98],
176: [151, 105, 91],
178: [180, 132, 85],
179: [137, 135, 136],
180: [215, 169, 75],
190: [249, 214, 46],
191: [232, 171, 45],
192: [105, 64, 39],
193: [207, 96, 36],
194: [163, 162, 164],
195: [70, 103, 164],
196: [35, 71, 139],
198: [142, 66, 133],
199: [99, 95, 97],
200: [130, 138, 93],
208: [229, 228, 222],
209: [176, 142, 68],
210: [112, 149, 120],
211: [121, 181, 181],
212: [159, 195, 233],
213: [108, 129, 183],
216: [143, 76, 42],
217: [124, 92, 69],
218: [150, 112, 159],
219: [107, 98, 155],
220: [167, 169, 206],
221: [205, 98, 152],
222: [228, 173, 200],
223: [220, 144, 149],
224: [240, 213, 160],
225: [235, 184, 127],
226: [253, 234, 140],
232: [125, 187, 221],
268: [52, 43, 117],
40: [236, 236, 236],
41: [205, 84, 75],
42: [193, 223, 240],
43: [123, 182, 232],
44: [247, 241, 141],
47: [217, 133, 108],
48: [132, 182, 141],
49: [248, 241, 132],
50: [236, 232, 222],
111: [191, 183, 177],
113: [228, 173, 200],
126: [165, 165, 203],
133: [213, 115, 61],
134: [216, 221, 86],
143: [207, 226, 247],
157: [255, 246, 123],
158: [225, 164, 194],
176: [151, 105, 91],
178: [180, 132, 85],
179: [137, 135, 136],
418: [0,191,89],
494: [204,204,204],
503: [230,227,218]
}


global childList              #list of child meshes for a parent mesh
childList={'first':[1,1]}
childList.clear()
global nulMat, nulOff

nullMat=Blender.Mathutils.Matrix(
                  [float(1), float(0), float(0)],
                  [float(0), float(1), float(0)],
                  [float(0), float(0), float(1)])
nullOff=Vector([float(0), float(0), float(0)])

#////////////////////////////////// SPLIT ////////////////////////////////////*
#/////
#///// Returns a list of all the word of the passed string
#/////
#////////////////////////////////// SPLIT ////////////////////////////////////*
def Split(string):
    l = []
    if len(string) == 0:
        return l
    if string[0] == '\n' or string[0] == '\r':
        return l
    i = 0
    while i < len(string):
        if string[i] == ' '  or string[i] == '\t' or \
           string[i] == '\n' or string[i] == '\r':
            if i != 0 :
                l = l + [string[0:i]]
            return l + Split(string[i+1:])
        i = i + 1
    return l
#//////////////////////////////////////////////////////////////////////////////
#///// Check whether a file is a part/model file (in PARTS or MODELS) dir.
#//////////////////////////////////////////////////////////////////////////////
def isPart(cmd, aPath=""):
  if len(cmd) != 15:      # Check for correct number of parameters
        return 0
  PartName = cmd[14]
  # Search for part in current import directory
  #strFileName = aPath + sys.sep + PartName
   
  strFileName = LDrawPath + sys.sep + "PARTS" + sys.sep + PartName
  try:
   
    aFile = open(strFileName, "rb")
    res=1
  except IOError:
    strFileName = LDrawPath + sys.sep + "MODELS" + sys.sep + PartName   
    try:
      aFile = open(strFileName, "rb")
      return 1
    except IOError:
      return 0 
    return 0

  aFile.close()
  return res

####################################
## multiplies 2 matrises
####################################
def doubleTransformMat(M,M1):
  result=Matrix(
         [(M1[0][0]*M[0][0]+M1[0][1]*M[1][0]+M1[0][2]*M[2][0]),
          (M1[0][0]*M[0][1]+M1[0][1]*M[1][1]+M1[0][2]*M[2][1]),
          (M1[0][0]*M[0][2]+M1[0][1]*M[1][2]+M1[0][2]*M[2][2])],         
         [(M1[1][0]*M[0][0]+M1[1][1]*M[1][0]+M1[1][2]*M[2][0]),
          (M1[1][0]*M[0][1]+M1[1][1]*M[1][1]+M1[1][2]*M[2][1]),
          (M1[1][0]*M[0][2]+M1[1][1]*M[1][2]+M1[1][2]*M[2][2])],
         [(M1[2][0]*M[0][0]+M1[2][1]*M[1][0]+M1[2][2]*M[2][0]),
          (M1[2][0]*M[0][1]+M1[2][1]*M[1][1]+M1[2][2]*M[2][1]),
          (M1[2][0]*M[0][2]+M1[2][1]*M[1][2]+M1[2][2]*M[2][2])])
  return result

###########################################
## calculates v+M*v1
#########################################

def doubleTransformVect(v,v1,M):
  ir=myMatMultVec(M,v1)
  result=Vector([(v.x+ir.x),(v.y+ir.y),(v.z+ir.z)]);
  return result

##################################
## my own MatMultVec
## the one in mathutils didn't work for me
#########################
def myMatMultVec(M,V):
 result=Vector([M[0][0]*V[0]+M[1][0]*V[1]+M[2][0]*V[2],
               M[0][1]*V[0]+M[1][1]*V[1]+M[2][1]*V[2],
               M[0][2]*V[0]+M[1][2]*V[1]+M[2][2]*V[2]])
 return result
 



#####################################################
## add a primitive
###################################################
def AddPrim(aObj, aMesh, avList, cmd, oldOff,oldMat, aPath="", pcolor = 16,newObj=0):
    if cmd[0] != '1':       # Check if line contains correct command
        return
    if len(cmd) != 15:      # Check for correct number of parameters
        return
    color = int(cmd[1])
    if color == 16:
       color = pcolor
   
    PartName = cmd[14]
    # Search for part in current import directory
    strFileName = aPath + sys.sep + PartName
    try:
        aFile = open(strFileName, "rb")
    except IOError:
        # Search for part in Ldraw PARTS directory
        strFileName = LDrawPath + sys.sep + "PARTS" + sys.sep + PartName
        try:
            aFile = open(strFileName, "rb")
        except IOError:
            # Search for part in Ldraw P directory
            strFileName = LDrawPath + sys.sep + "P" + sys.sep + PartName
            try:
                aFile = open(strFileName, "rb")
            except IOError:
                print("Unable to find part file %s!" % (PartName))
                return
    aFile.close()
   
    ImportDAT(2,strFileName, aObj, aMesh, avList,oldOff,oldMat, color)
    
    return

#########################################
## ad a part
####################################
def AddPart(aObj, aMesh, avList, cmd, off,tranfMatrix, aPath="", pcolor = 16,newObj=0):
    if cmd[0] != '1':       # Check if line contains correct command
        return
    if len(cmd) != 15:      # Check for correct number of parameters
        return
    color = int(cmd[1])
    if color == 16:
       color = pcolor
   
    PartName = cmd[14]
    # Search for part in current import directory
    strFileName = aPath + sys.sep + PartName
    try:
        aFile = open(strFileName, "rb")
    except IOError:
        # Search for part in Ldraw PARTS directory
        strFileName = LDrawPath + sys.sep + "PARTS" + sys.sep + PartName
        try:
            aFile = open(strFileName, "rb")
        except IOError:
            # Search for part in Ldraw P directory
            strFileName = LDrawPath + sys.sep + "P" + sys.sep + PartName
            try:
                aFile = open(strFileName, "rb")
            except IOError:
                print("Unable to find part file %s!" % (PartName))
                return
    aFile.close()
    ImportDAT(1,strFileName, aObj, aMesh, avList,off,tranfMatrix, color)
       
    return

######################################################
### transfor a vector to a new coord-system
#############################################
def transVertex(vert,ofs,mtr):
    vec=Vector([vert[0],vert[1],vert[2]])
    v1=myMatMultVec(mtr,vec)
    vertList=[v1.x+ofs.x,v1.y+ofs.y,v1.z+ofs.z]
    result=NMesh.Vert(vertList[0],vertList[1],vertList[2])
    return result


#########################################
## check whether vertex already exists in this mesh
#############################
def checkVert(v,vList,aMesh):
    vert=v.co
    vertL=[vert[0],vert[1],vert[2]]
    if vertL in vList:
       idx1=vList.index(vertL)
    else:
       aMesh.verts.append(v)
       vList.append(vertL)
       idx1=len(aMesh.verts)-1
    return idx1



#////////////////////////////////// ADDTRIANGLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\*
#/////
#/////	Extract from the string 'aLine' vertex information of the
#/////    Triangle primitive, create the face and add it to the Mesh
#/////    object passed 'aMesh'
#/////
#////////////////////////////////// ADDTRIANGLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\*
def AddTriangle( aMesh, vList, cmd,offset,tm):
    if cmd[0] != '3':       # Check if line contains correct command
        return
    if len(cmd) != 11:      # Check for correct number of parameters
        return
   
    f = NMesh.Face()
    for x in range(3):
        v = NMesh.Vert(float(cmd[2+(x*3)]), float(cmd[3+(x*3)]), float(cmd[4+(x*3)]))
        v=transVertex(v,offset,tm)
        idx=checkVert(v,vList,aMesh)  
        f.v.append(aMesh.verts[idx])
    f.smooth=1
    aMesh.faces.append(f)
    return   

#/////////////////////////////// ADDQUADRILATERAL \\\\\\\\\\\\\\\\\\\\\\\\\\*
#/////
#/////	Extract from the string 'aLine' vertex information of the
#/////    Quadrilateral primitive, create the face and add it to the Mesh
#/////    object passed 'aMesh'
#/////
#/////////////////////////////// ADDQUADRILATERAL \\\\\\\\\\\\\\\\\\\\\\\\\\*
def AddQuadrilateral( aMesh, vList, cmd,offset,tm):
    #print "quad"
    if cmd[0] != '4':       # Check if line contains correct command
        return
    if len(cmd) != 14:      # Check for correct number of parameters
        return

    f = NMesh.Face()
    for x in range(4):
        v = NMesh.Vert(float(cmd[2+(x*3)]), float(cmd[3+(x*3)]), float(cmd[4+(x*3)]))
        v=transVertex(v,offset,tm)
        idx=checkVert(v,vList,aMesh)
        f.v.append(aMesh.verts[idx])
    f.smooth=1
    aMesh.faces.append(f)
    return

#////////////////////////////////// IMPORTDAT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
#/////
#/////	Main function of the class, open file and import the LDraw File
#/////
#////////////////////////////////// IMPORTDAT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
def ImportDAT(type,strFileName ,aObj, aMesh ,avList,offs, transMatrix, pcolor = 16 ):
    #aMesh = []
    mFile = []         # File pointer
    
    Line = ""
    cmdLine = []
    aPath = sys.dirname(strFileName)
    aName = uppercase(sys.basename(strFileName))

    # Open the DAT file
    try:
        mFile = open(strFileName, "rb")
    except IOError:
        # Make sure we have a valid file pointer (we found the file)
        print("Unable to find the file %s!" % (strFileName))
        return
  
    # Once the file is opened, we read all the lines
    Line = mFile.readline()
    while Line <> "":
         cmdLine = Split(Line)
         if cmdLine == []:
             pass
         elif cmdLine[0]=='0':  # Remark
             pass
         elif cmdLine[0]=='1':  # Part File
              newMat = Matrix([float(cmdLine[ 5]), float(cmdLine[ 8]), float(cmdLine[11])],
                                    [float(cmdLine[ 6]), float(cmdLine[ 9]), float(cmdLine[12])],
                                    [float(cmdLine[ 7]), float(cmdLine[10]), float(cmdLine[13])])
              newOff=Vector([float(cmdLine[ 2]), float(cmdLine[ 3]), float(cmdLine[ 4])])
                 
              oldMat=transMatrix
              partMatrix=doubleTransformMat(oldMat,newMat)
              oldoff=offs
              partOff=doubleTransformVect(oldoff,newOff,oldMat)
              tfMat=Matrix([partMatrix[0][0],partMatrix[0][1],partMatrix[0][2],0],
                                [partMatrix[1][0],partMatrix[1][1],partMatrix[1][2],0],
                                [partMatrix[2][0],partMatrix[2][1],partMatrix[2][2],0],
                                [partOff[0],partOff[1],partOff[2],1])
              if isPart(cmdLine,aPath)==1:
                
                 aName=cmdLine[14]
                 print "part: ", aName
                 bObj = Blender.Object.New('Mesh',aName)
                 oldMesh=Blender.NMesh.GetRaw(aName)
                 scene = Blender.Scene.getCurrent ()
                 bObj.setMatrix(tfMat)   
                 if oldMesh!=None:
                       bMesh=oldMesh
                       bObj.link(bMesh)
                       scene.link (bObj)
                       if childList.has_key(bMesh.name):
                          for kid in childList[bMesh.name]:
                              kidObj = Blender.Object.New('Mesh',kid[0].name) 
                              oM=partMatrix
                              kM=doubleTransformMat(oM,kid[1])
                              oO=partOff
                              kO=doubleTransformVect(oO,kid[2],oM)
                              kidMat=Matrix([kM[0][0],kM[0][1],kM[0][2],0],
                                [kM[1][0],kM[1][1],kM[1][2],0],
                                [kM[2][0],kM[2][1],kM[2][2],0],
                                [kO[0],kO[1],kO[2],1])
                              kidObj.setMatrix(kidMat)
                              kidObj.link(kid[0])
                              scene.link (kidObj)
                              bObj.makeParent([kidObj]) 
                 else:
                       
                       bMesh = Blender.NMesh.New(aName)
                       
                       bMesh.addMaterial(Material.Get("0"))
                       bObj.link(bMesh)
                       scene.link (bObj)
                       bObj.setMatrix(tfMat)
                       bvList=[]
                       AddPart(bObj, bMesh, bvList, cmdLine,partOff,partMatrix, aPath, pcolor,1)
                       bMesh.update()
                        
                 color=int(cmdLine[1])
                 try:
                      legocolor[color]
                      mat = Material.Get(cmdLine[1])
                      bObj.setMaterials([mat])
                      bObj.colbits=1
                      
                 except KeyError:
                      print ""
                    
                 bMesh.update()
                 aObj.makeParent([bObj])
                          
                 if childList.has_key(aMesh.name):
                   childeren=childList[aMesh.name]
                   if childeren!=None:
                      childeren.append([bMesh,newMat, newOff])
                      childList[aMesh.name]=childeren
                   else:
                      childList[aMesh.name]=[[bMesh,newMat, newOff]]
                 else:
                  
                   childList[aMesh.name]=[[bMesh,newMat, newOff]]
                          
              else:
                 #print "is primitief",aName
                 if type==1:
                    AddPrim(aObj,aMesh, avList, cmdLine, newOff,newMat, aPath,pcolor,0)
                 else:
                    AddPrim(aObj,aMesh, avList, cmdLine, partOff,partMatrix, aPath,pcolor,0)
                 aMesh.update()
         elif cmdLine[0]=='2':  # Line
             pass
         elif cmdLine[0]=='3':  # Triangle
             #print "tri: "
             if type==1:
                AddTriangle(aMesh,avList, cmdLine,nullOff, nullMat)
             else:
                AddTriangle(aMesh,avList, cmdLine,offs, transMatrix)
         elif cmdLine[0]=='4':  # Quadrilateral
             #print "quad: "
             if type==1:
                 AddQuadrilateral(aMesh,avList, cmdLine, nullOff, nullMat)
             else:
                 AddQuadrilateral(aMesh,avList, cmdLine, offs, transMatrix)
         elif cmdLine[0]=='5':  # Optional-Line
             pass
         else :
             pass
         Line = mFile.readline()
    aMesh.setMaxSmoothAngle(45)
    aMesh.setMode("AutoSmooth")
    mFile.close()
    # If the mesh contains vertex then link this mesh in the object
    aMesh.update()
    

    return


#////////////////////////////////// LOWERCASE //////////////////////////////////*
#/////
#///// This function set all character of the string into lowercase chars
#/////
#///////////////////////////////// LOWERCASE ///////////////////////////////////*
def lowercase(s):
    ret = ""
    i = 0
    while i < len(s):
        if s[i] >= 'A' and s[i] <= 'Z':
            ret = ret + chr(ord(s[i])+ord('a')-ord('A'))
        else :
            ret = ret + s[i]
        i = i + 1
    return ret


#////////////////////////////////// UPPERCASE //////////////////////////////////*
#/////
#///// This function set all character of the string into uppercase chars
#/////
#///////////////////////////////// UPPERCASE ///////////////////////////////////*
def uppercase(s):
    ret = ""
    i = 0
    while i < len(s):
        if s[i] >= 'a' and s[i] <= 'z':
            ret = ret + chr(ord(s[i])+ord('A')-ord('a'))
        else :
            ret = ret + s[i]
        i = i + 1
    return ret


#//////////////////////////////////// EXITGUI /////////////////////////////////*
#/////
#/////	Exit form the gui
#/////
#//////////////////////////////////// EXITGUI /////////////////////////////////*
def ExitGUI ():
        Exit()

#//////////////////////////////////// EVENTGUI /////////////////////////////////*
#/////
#/////	Process events
#/////
#//////////////////////////////////// EVENTGUI /////////////////////////////////*
def	EventGUI (event):
        global bMaterials
	if (event==EVENT_IMPORT):
                bMaterials = False
                Blender.Window.FileSelector(Import, "LDRAW Import")
	elif (event==EVENT_QUIT):
                ExitGUI()

#//////////////////////////////////// DRAWGUI /////////////////////////////////*
#/////
#/////	Draw the gui in blender
#/////
#//////////////////////////////////// DRAWGUI /////////////////////////////////*
def DrawGUI():

	glClearColor(0.6,0.6,0.6,0)
	glClear(GL_COLOR_BUFFER_BIT)
	glRasterPos2d(100,64)
	Text("Import LDRAW (Lego CAD) file")
	Button ("Import",EVENT_IMPORT,32,10,100,32,"Select File to Import")
	Button ("Cancel",EVENT_QUIT,252,10,100,32,"Do not Import")


#//////////////////////////////////// IMPORT /////////////////////////////////*
#/////
#/////	Load the 3ds file, parse it and import everything in blender
#/////
#//////////////////////////////////// IMPORT /////////////////////////////////*
def Import(FileName):
#DBG        print "--------------- START IMPORT ----------------"
        scene = Blender.Scene.getCurrent ()

        # Create Lego Materials
        mName = uppercase(sys.basename(FileName))
        for i,LdrawColor in legocolor.iteritems():
            mat = Blender.Material.New(str(i))
            mat.rgbCol = [float(LdrawColor[0])/255,
                          float(LdrawColor[1])/255,
                          float(LdrawColor[2])/255]
            #mat.setAlpha( float(LdrawColor[i][3])/255.0)
            
        aObj = Blender.Object.New('Mesh',mName)
        aMesh = Blender.NMesh.New(mName)
        aObj.link(aMesh)
        scene.link (aObj)
        aObj.setLocation(0,0,0)
        tranfMatrix =nullMat 
        offst=nullOff
        avList=[]
        print "-----------------------------------------------"
        try:
          ImportDAT(1,FileName,aObj, aMesh,avList,offst,tranfMatrix,16)
        except IOError:
          Draw()
          print "error while loading Ldraw file make sure path is correct"
        aMesh.update()
        corrMat=Matrix([1,0,0,0],
                       [0,0,-1,0],
                       [0,1,0,0],
                       [0,0,0,1])
        aObj.setMatrix(corrMat)
        Blender.Redraw()
        print "-------------------FINISHED-----------------------"


#//////////////////////////////////// REGISTERGUI /////////////////////////////////*
#/////
#/////   Register callbacks for windowing
#/////
#/////   DrawGui  : A function to draw the screen, taking no arguments
#/////   None     : No function to handle keyboard and mouse input events
#/////   EventGUI : A function to handle Draw Button events, taking 'event' argument
#/////
#//////////////////////////////////// REGISTERGUI /////////////////////////////////*
def RegisterGUI ():
	Register (DrawGUI,None,EventGUI)

#////////////////////////////////////  MAIN SCRIPT /////////////////////////////////*
#/////
#/////	Start the script
#/////
#////////////////////////////////////  MAIN SCRIPT /////////////////////////////////*
RegisterGUI()
